#ifndef QUDEVDEVICE_H
#define QUDEVDEVICE_H

#include <QHash>
#include <QObject>
#include <QSharedPointer>
#include <QVariant>

#include "Enums.h"

#include "QUdev_Global.h"

class QUdev;
class QUdevContext;
class QUdevDeviceData;

typedef QHash<QString, QVariant> QUdevProperties;
typedef unsigned long long int seqnum_t;
typedef unsigned long long int usec_t;

class QUDEVSHARED_EXPORT QUdevDevice : public QObject
{
public:
    QUdevDevice(const QUdevDevice &other);
    virtual ~QUdevDevice();

    QUdevDevice &operator=(const QUdevDevice &other);

    QUdevDevice parentDevice(const QString &susbsystem = QString(), const QString &devtype = QString()) const;
    QUdevProperties properties() const;
    QStringList tagList() const;
    QStringList sysattrList() const;
    QString action() const;
    QString devnode() const;
    QString devpath() const;
    QString driver() const;
    QString subsystem() const;
    QString sysname() const;
    QString sysnum() const;
    QString syspath() const;
    QString type() const;
    QVariant value(const QString &property, bool decode = true) const;
    QVariant value(QUdevNS::Property property, bool decode = true) const;
    bool isValid() const;
    bool isInitialized() const;
    seqnum_t seqnum() const;
    usec_t timeSinceInitialized() const;
    void printInfo() const;

    static QUdevDevice fromSyspath(const QUdev &udev, const QString &syspath, QObject *parent = 0);
    static QUdevDevice fromSubsystem(const QUdev &udev, const QString &subsystem, const QString &sysname, QObject *parent = 0);
    static QUdevDevice fromSubsystem(const QUdev &udev, QUdevNS::Subsystem subsystem, const QString &sysname, QObject *parent = 0);

protected:
    QUdevDevice(const QUdevContext &context, struct udev_device *device, QObject *parent = 0);
    friend class QUdev;
    friend class QUdevMonitor;

private:
    QSharedDataPointer<QUdevDeviceData> d;
};

#endif // QUDEVDEVICE_H
