#ifndef QUDEVENUMERATOR_H
#define QUDEVENUMERATOR_H

#include <QList>
#include <QObject>
#include <QStringList>

#include "QUdev_Global.h"
#include "QUdevDevice.h"

class QUdev;
class QUdevEnumeratorPrivate;

typedef QList<QUdevDevice> QUdevDeviceList;

class QUDEVSHARED_EXPORT QUdevEnumerator : public QObject
{
public:
    enum FilterOption
    {
        Match = 0,
        DoNotMatch
    };

    enum ScanOption
    {
        ScanAllDevices = 0,
        ScanSubsystems
    };

    explicit QUdevEnumerator(const QUdev &udev, QObject *parent = 0);
    virtual ~QUdevEnumerator();

    bool addFilterSubsystem(const QString &filter, FilterOption filterOption = Match);
    bool addFilterSubsystem(QUdevNS::Subsystem filter, FilterOption filterOption = Match);
    bool addFilterSysattr(const QString &attribute, const QString &value = QLatin1String("*"), FilterOption filterOption = Match);
    bool addFilterProperty(const QString &property, const QString &value = QLatin1String("*"));
    bool addFilterProperty(QUdevNS::Property property, const QString &value = QLatin1String("*"));
    bool addFilterTag(const QString &filter);
    bool addFilterSysname(const QString &filter);
    bool addSyspath(const QString &syspath);
    bool reset();

    QStringList listDevicePaths(ScanOption scanOption = ScanAllDevices) const;
    QUdevDeviceList listDevices(ScanOption scanOption = ScanAllDevices) const;

private:
    Q_DISABLE_COPY(QUdevEnumerator)
    Q_DECLARE_PRIVATE(QUdevEnumerator)
    QUdevEnumeratorPrivate *d_ptr;
};

#endif // QUDEVENUMERATOR_H
