#ifndef QUDEVMONITOR_H
#define QUDEVMONITOR_H

#include <QObject>

#include "QUdev_Global.h"
#include "QUdevDevice.h"

class QUdev;
class QUdevMonitorPrivate;

class QUDEVSHARED_EXPORT QUdevMonitor : public QObject
{
    Q_OBJECT
public:
    enum DeviceAction
    {
        DeviceAdded = 0,
        DeviceRemoved,
        DeviceChanged,
        DeviceOnline,
        DeviceOffline,
        DeviceOtherAction
    };

    enum EventSource
    {
        Udev = 0,
        Kernel
    };

    explicit QUdevMonitor(const QUdev &udev, EventSource eventSource = Udev, QObject *parent = 0);
    virtual ~QUdevMonitor();

    bool isListening() const;
    bool reset();
    bool addFilterSubsystem(const QString &subsystem, const QString &devtype = QString()) const;
    bool addFilterSubsystem(QUdevNS::Subsystem subsystem, const QString &devtype = QString()) const;
    bool clearFilters() const;

public slots:
    bool startListening();
    bool stopListening();

signals:
    void deviceAdded(const QString &devicePath);
    void deviceRemoved(const QString &devicePath);
    void deviceChanged(const QString &devicePath);
    void deviceOnline(const QString &devicePath);
    void deviceOffline(const QString &devicePath);

    void deviceAdded(QUdevDevice device);
    void deviceRemoved(QUdevDevice device);
    void deviceChanged(QUdevDevice device);
    void deviceOnline(QUdevDevice device);
    void deviceOffline(QUdevDevice device);

    void deviceChanged(QUdevDevice device, int action);

protected slots:
    void processEvent();

private:
    Q_DISABLE_COPY(QUdevMonitor)
    Q_DECLARE_PRIVATE(QUdevMonitor)
    QUdevMonitorPrivate *d_ptr;
};

#endif // QUDEVMONITOR_H
